package com.manimarank.websitemonitor.utils

import android.annotation.SuppressLint
import android.app.*
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.PowerManager
import android.provider.Settings
import android.view.View
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatDelegate
import androidx.core.app.NotificationCompat
import com.google.android.material.snackbar.Snackbar
import com.manimarank.websitemonitor.MyApplication
import com.manimarank.websitemonitor.R
import com.manimarank.websitemonitor.data.model.WebSiteStatus
import com.manimarank.websitemonitor.ui.home.MainActivity
import com.manimarank.websitemonitor.utils.Constants.DEFAULT_INTERVAL_MIN
import com.manimarank.websitemonitor.utils.Constants.NOTIFICATION_CHANNEL_DESCRIPTION
import com.manimarank.websitemonitor.utils.Constants.NOTIFICATION_CHANNEL_ID
import com.manimarank.websitemonitor.utils.Constants.NOTIFICATION_CHANNEL_NAME
import com.manimarank.websitemonitor.utils.Interval.nameList
import com.manimarank.websitemonitor.utils.Interval.valueList
import com.manimarank.websitemonitor.utils.SharedPrefsManager.get
import com.manimarank.websitemonitor.utils.SharedPrefsManager.set
import com.manimarank.websitemonitor.worker.WorkManagerScheduler
import java.net.HttpURLConnection
import java.net.URL
import java.text.DateFormat
import java.util.*

object Utils {

    val lineEnd = System.lineSeparator()
    var totalAmountEntry = 0

    fun currentDateTime(): String {
        return DateFormat.getDateTimeInstance().format(Date())
    }

    fun showNotification(context: Context, title: String, message: String) {
        val mNotificationManager = context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            val channel = NotificationChannel(NOTIFICATION_CHANNEL_ID,
                NOTIFICATION_CHANNEL_NAME,
                NotificationManager.IMPORTANCE_HIGH)
            channel.description = NOTIFICATION_CHANNEL_DESCRIPTION
            mNotificationManager.createNotificationChannel(channel)
        }
        val mBuilder = NotificationCompat.Builder(context, NOTIFICATION_CHANNEL_ID)
            .setSmallIcon(R.drawable.ic_alert) // notification icon
            .setContentTitle(title) // title for notification
            .setContentText(message)// message for notification
            .setDefaults(Notification.DEFAULT_SOUND)
            .setAutoCancel(true) // clear notification after click

        val intent = Intent(context, MainActivity::class.java)
        intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
        val pi = PendingIntent.getActivity(context, 0, intent, PendingIntent.FLAG_IMMUTABLE)
        mBuilder.setContentIntent(pi)
        mNotificationManager.notify(Random().nextInt(), mBuilder.build())
    }

    fun isValidUrl(url: String) : Boolean{
        try {
            URL(url).toURI()
            return true
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return false
    }

    fun startWorkManager(context: Context, isForce : Boolean = false) {
        val isScheduled: Boolean? = SharedPrefsManager.customPrefs[Constants.IS_SCHEDULED, false]

        isScheduled?.let { scheduled ->
            if (!scheduled || isForce) {
                SharedPrefsManager.customPrefs.set(Constants.IS_SCHEDULED, true)
                WorkManagerScheduler.refreshPeriodicWork(context)
            }
        }
    }

    fun getMonitorInterval() : Long {
        return (SharedPrefsManager.customPrefs[Constants.MONITORING_INTERVAL, DEFAULT_INTERVAL_MIN] ?: DEFAULT_INTERVAL_MIN).toLong()
    }

    fun getMonitorTime() : String {
        val interval = getMonitorInterval().toInt()
        var refreshTime: String? = null
        if (valueList.contains(interval)) {
            val pos = valueList.indexOf(interval)
            if (pos >= 0 && pos < nameList.size)
                refreshTime = nameList[pos]
        }
        return "Checking every ${refreshTime ?: "1 hour once"}"
    }

    fun isCustomRom(): Boolean {
        return listOf("xiaomi", "oppo", "vivo", "letv", "honor")
            .contains(
                Build.MANUFACTURER.lowercase(Locale.ROOT)
            )
    }

    @SuppressLint("QueryPermissionsNeeded")
    fun openAutoStartScreen(context: Context) {
        try {
            val intent = Intent()
            // https://stackoverflow.com/questions/39366231/how-to-check-miui-autostart-permission-programmatically
            when(Build.MANUFACTURER.lowercase(Locale.ROOT)) {
                "xiaomi" -> intent.component= ComponentName("com.miui.securitycenter", "com.miui.permcenter.autostart.AutoStartManagementActivity")
                "oppo" -> intent.component = ComponentName("com.coloros.safecenter", "com.coloros.safecenter.permission.startup.StartupAppListActivity")
                "vivo" -> intent.component = ComponentName("com.vivo.permissionmanager", "com.vivo.permissionmanager.activity.BgStartUpManagerActivity")
                "letv" -> intent.component = ComponentName("com.letv.android.letvsafe", "com.letv.android.letvsafe.AutobootManageActivity")
                "honor" -> intent.component = ComponentName("com.huawei.systemmanager", "com.huawei.systemmanager.optimize.process.ProtectActivity")
            }

            val list = context.packageManager.queryIntentActivities(intent, PackageManager.MATCH_DEFAULT_ONLY)
            if (list.size > 0) {
                context.startActivity(intent)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            Toast.makeText(context, context.getString(R.string.something_went_wrong), Toast.LENGTH_LONG).show()
        }

    }

    fun showAutoStartEnableDialog(context: Context) {
        if (isCustomRom() && !SharedPrefsManager.customPrefs.getBoolean(Constants.IS_AUTO_START_SHOWN, false)) {
            val alertBuilder = AlertDialog.Builder(context)
            alertBuilder.setTitle(context.getString(R.string.enable_auto_start))
            alertBuilder.setMessage(context.getString(R.string.message_auto_start_reason))
            alertBuilder.setPositiveButton(context.getString(R.string.ok)) { dialog, _ ->
                SharedPrefsManager.customPrefs[Constants.IS_AUTO_START_SHOWN] = true
                openAutoStartScreen(context)
                dialog.dismiss()
            }
            alertBuilder.setNegativeButton(context.getString(R.string.cancel), null)
            val dialog = alertBuilder.create()
            dialog.setCancelable(false)
            dialog.show()
        }
    }

    object RequestCode {
        const val RC_IGNORE_BATTERY_OPTIMIZATION = 101
    }

    @SuppressLint("BatteryLife")
    fun Activity.askToRunBackground(requestCode: Int = RequestCode.RC_IGNORE_BATTERY_OPTIMIZATION) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            val pm = this.getSystemService(Context.POWER_SERVICE) as PowerManager
            val isIgnoringBatteryOptimizations = pm.isIgnoringBatteryOptimizations(packageName)
            if (!isIgnoringBatteryOptimizations) {
                val intent = Intent().apply {
                    action = Settings.ACTION_REQUEST_IGNORE_BATTERY_OPTIMIZATIONS
                    data = Uri.parse("package:$packageName")
                }
                startActivityForResult(intent, requestCode)
            }
        }
    }

    fun openUrl(context: Context, url: String) {
        try {
            val intents = Intent(Intent.ACTION_VIEW)
            intents.data = Uri.parse(url)
            intents.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            context.startActivity(intents)
        } catch (e: Exception) {
            Print.log(e.toString())
            Toast.makeText(context, context.getString(R.string.no_apps_found), Toast.LENGTH_LONG)
                .show()
        }
    }

    fun showToast(context: Context, message: String) {
        Toast.makeText(context, message, Toast.LENGTH_LONG).show()
    }

    fun showSnackBar(view: View, message: String) {
        Snackbar.make(view, message, Snackbar.LENGTH_LONG).show()
    }

    val statusCodesList get() = hashMapOf<Int, String>().apply {
        this[HttpURLConnection.HTTP_OK] = "Success"
        this[HttpURLConnection.HTTP_CREATED] = "Created" // 201
        this[HttpURLConnection.HTTP_ACCEPTED] = "Accepted" // 202
        this[HttpURLConnection.HTTP_NOT_AUTHORITATIVE] = "Non-Authoritative Information" // 203
        this[HttpURLConnection.HTTP_NO_CONTENT] = "No Content" // 204
        this[HttpURLConnection.HTTP_RESET] = "Reset Content" // 205
        this[HttpURLConnection.HTTP_PARTIAL] = "Partial Content" // 206
        this[HttpURLConnection.HTTP_MULT_CHOICE] = "Multiple Choices" // 300
        this[HttpURLConnection.HTTP_MOVED_PERM] = "Moved Permanently" // 301
        this[HttpURLConnection.HTTP_MOVED_TEMP] = "Temporary Redirect" // 302
        this[HttpURLConnection.HTTP_SEE_OTHER] = "See Other" // 303
        this[HttpURLConnection.HTTP_NOT_MODIFIED] = "Not Modified" // 304
        this[HttpURLConnection.HTTP_USE_PROXY] = "Use Proxy" // 305
        this[HttpURLConnection.HTTP_BAD_REQUEST] = "Bad Request" // 400
        this[HttpURLConnection.HTTP_UNAUTHORIZED] = "Unauthorized" // 401
        this[HttpURLConnection.HTTP_PAYMENT_REQUIRED] = "Payment Required" // 402
        this[HttpURLConnection.HTTP_FORBIDDEN] = "Forbidden" // 403
        this[HttpURLConnection.HTTP_NOT_FOUND] = "Not Found" // 404
        this[HttpURLConnection.HTTP_BAD_METHOD] = "Method Not Allowed" // 405
        this[HttpURLConnection.HTTP_NOT_ACCEPTABLE] = "Not Acceptable" // 406
        this[HttpURLConnection.HTTP_PROXY_AUTH] = "Proxy Authentication Required" // 407
        this[HttpURLConnection.HTTP_CLIENT_TIMEOUT] = "Request Time-Out" // 408
        this[HttpURLConnection.HTTP_CONFLICT] = "Conflict" // 409
        this[HttpURLConnection.HTTP_GONE] = "Gone" // 410
        this[HttpURLConnection.HTTP_LENGTH_REQUIRED] = "Length Required" // 411
        this[HttpURLConnection.HTTP_PRECON_FAILED] = "Precondition Failed" // 412
        this[HttpURLConnection.HTTP_ENTITY_TOO_LARGE] = "Request Entity Too Large" // 413
        this[HttpURLConnection.HTTP_REQ_TOO_LONG] = "Request-URI Too Large" // 414
        this[HttpURLConnection.HTTP_UNSUPPORTED_TYPE] = "Unsupported Media Type" // 415
        this[HttpURLConnection.HTTP_INTERNAL_ERROR] = "Internal Server Error" // 500
        this[HttpURLConnection.HTTP_NOT_IMPLEMENTED] = "Not Implemented" // 501
        this[HttpURLConnection.HTTP_BAD_GATEWAY] = "Bad Gateway" // 502
        this[HttpURLConnection.HTTP_UNAVAILABLE] = "Service Unavailable" // 503
        this[HttpURLConnection.HTTP_GATEWAY_TIMEOUT] = "Gateway Timeout" // 504
        this[HttpURLConnection.HTTP_VERSION] = "HTTP Version Not Supported" // 505
    };

    fun getStatusMessage(code: Int?): String {
        return if (statusCodesList.contains(code)) statusCodesList[code] ?: "" else "Unknown"
    }

    private fun isServerRelatedFail(status: Int): Boolean {
        return status >= 500
    }

    fun monitorStatusCodes(): List<String>? {
        return SharedPrefsManager.customPrefs.getStringSet(Constants.MONITORING_STATUS_CODES, null)?.sorted()
    }

    private fun isMonitorStatusCode(status: Int): Boolean {
        val trackedStatusCodes = monitorStatusCodes()
        return trackedStatusCodes == null || trackedStatusCodes.contains("$status")
    }

    fun mayNotifyStatusFailure(status: Int): Boolean {
        return status != HttpURLConnection.HTTP_OK && isMonitorStatusCode(status)
    }

    fun Context.getStringNotWorking(url: String): String {
        return String.format(
            this.getString(
                R.string.not_working,
                url
            )
        )
    }

    fun List<WebSiteStatus>.joinToStringDescription(): String {
        return this.joinToString(lineEnd) { status ->
            status.name
        }
    }

    fun resumeApp() {
        MyApplication.ActivityVisibility.resumeApp()
    }

    fun pauseApp() {
        MyApplication.ActivityVisibility.pauseApp()
    }

    fun appIsVisible(): Boolean {
        return MyApplication.ActivityVisibility.appIsVisible
    }

    fun String.removeUrlProto(): String = this.replace(Regex("""^http[s]?://"""), "")

    fun enableDarkMode(isDarkModeEnabled: Boolean) {
        AppCompatDelegate.setDefaultNightMode(if (isDarkModeEnabled) AppCompatDelegate.MODE_NIGHT_YES else AppCompatDelegate.MODE_NIGHT_NO)
    }
}